/*
# Revised for AT24C256 and my LPC1768 borad.
# (AT24C256 is I2C EEPROM with 32kbyte(8bit))
# Copyright (C) 2010, audin
# 2010/10
# http://avr.paslog.jp
# This file is licensed under the MIT License.
# Reference page:
# http://mbed.org/users/simon/programs/Test24LC256/431951f3d204eab8246d52bf6b468458/docs/main_8cpp_source.html
*/

// Test24LC256 hello world example
// Copyright (c) 2009, sford
// Released under the MIT License: http://mbed.org/license/mit
//
// Does some basic stuff with an 24LC256 to show it is working

#include "mbed.h"

#define BORAD_MBED	0		//if 1 then mbed board, else other board.

#if BORAD_MBED
I2C i2c(p28, p27);  		// sda, scl
DigitalOut wp(p29); 		// write protect
#else
I2C i2c(P0_27, P0_28);		// sda0, scl0 my borad
#endif

/* tx, rx    default Baudrate:9600bps
   p26=txd1=P2_0,  p25=rxd1=P2_1 */
Serial pc(P2_0, P2_1);
//Serial pc(USBTX, USBRX); // tx, rx

int main() {
  
#if BORAD_MBED
  wp = 0; // disable write protect
#endif
  pc.printf("Writing bytes 0-16\n");

  char data[3];
  for(int i=0; i<16; i++) {
	data[0] = 0;     // MSB address
	data[1] = i;     // LSB address
	data[2] = i * 3; // data
	if(i2c.write(0xA0, data, 3)) {
	  pc.printf("error:Write failed\n");
	}
	while(i2c.write(0xA0, NULL, 0)); // wait to complete
  }

  pc.printf("Setting read pointer to 0\n");

  data[0] = 0;  // MSB address
  data[1] = 0;  // LSB address
  if(i2c.write(0xA0, data, 2)) { // send address, but no data
	pc.printf("error:Write failed\n");
  }

  pc.printf("Reading back data bytes 0-16\n");

  char response[1];
  for(int i=0; i<16; i++) {
	if(i2c.read(0xA0, response, 1)) {
	  pc.printf("error:Read failed\n");
	}
	pc.printf("address %03d = 0x%02X\n", i, response[0]);
  }
}
